@@ -13,14 +13,14 @@ module VagrantPlugins
                 comm.execute("localcli storage nfs remove -v #{volume}")
               end
               mount_command = "localcli storage nfs add -H #{ip} -s '#{opts[:hostpath]}' -v '#{volume}'"
-              retryable(:on => Vagrant::Errors::LinuxNFSMountFailed, :tries => 5, :sleep => 2) do
+              retryable(on: Vagrant::Errors::LinuxNFSMountFailed, tries: 5, sleep: 2) do
                 comm.execute(mount_command,
-                             :error_class => Vagrant::Errors::LinuxNFSMountFailed)
+                             error_class: Vagrant::Errors::LinuxNFSMountFailed)
               end
 
               # symlink vmfs volume to :guestpath
               if comm.test("test -L '#{guestpath}'")
-                comm.execute("rm '#{guestpath}'")
+                comm.execute("rm -f '#{guestpath}'")
               end
               if comm.test("test -d '#{guestpath}'")
                 comm.execute("rmdir '#{guestpath}'")
