@@ -11,7 +11,9 @@ module VagrantPlugins
       attr_accessor :keep_alive
       attr_accessor :shell
       attr_accessor :proxy_command
+      attr_accessor :ssh_command
       attr_accessor :pty
+      attr_accessor :sudo_command
 
       attr_reader :default
 
@@ -23,8 +25,10 @@ module VagrantPlugins
         @guest_port    = UNSET_VALUE
         @keep_alive    = UNSET_VALUE
         @proxy_command = UNSET_VALUE
+        @ssh_command   = UNSET_VALUE
         @pty           = UNSET_VALUE
         @shell         = UNSET_VALUE
+        @sudo_command  = UNSET_VALUE
 
         @default    = SSHConnectConfig.new
       end
@@ -44,9 +48,14 @@ module VagrantPlugins
         @guest_port = 22 if @guest_port == UNSET_VALUE
         @keep_alive = true if @keep_alive == UNSET_VALUE
         @proxy_command = nil if @proxy_command == UNSET_VALUE
+        @ssh_command = nil if @ssh_command == UNSET_VALUE
         @pty        = false if @pty == UNSET_VALUE
         @shell      = "bash -l" if @shell == UNSET_VALUE
 
+        if @sudo_command == UNSET_VALUE
+          @sudo_command = "sudo -E -H %c"
+        end
+
         @default.username = "vagrant" if @default.username == UNSET_VALUE
         @default.port     = @guest_port if @default.port == UNSET_VALUE
         @default.finalize!
