@@ -7,12 +7,12 @@ require File.expand_path("../base", __FILE__)
 module VagrantPlugins
   module ProviderVirtualBox
     module Driver
-      # Driver for VirtualBox 4.1.x
-      class Version_4_1 < Base
+      # Driver for VirtualBox 4.2.x
+      class Version_4_2 < Base
         def initialize(uuid)
           super()
 
-          @logger = Log4r::Logger.new("vagrant::provider::virtualbox_4_1")
+          @logger = Log4r::Logger.new("vagrant::provider::virtualbox_4_2")
           @uuid = uuid
         end
 
@@ -28,8 +28,8 @@ module VagrantPlugins
         def clear_shared_folders
           info = execute("showvminfo", @uuid, "--machinereadable", retryable: true)
           info.split("\n").each do |line|
-            if folder = line[/^SharedFolderNameMachineMapping\d+="(.+?)"$/, 1]
-              execute("sharedfolder", "remove", @uuid, "--name", folder)
+            if line =~ /^SharedFolderNameMachineMapping\d+="(.+?)"$/
+              execute("sharedfolder", "remove", @uuid, "--name", $1.to_s)
             end
           end
         end
@@ -45,8 +45,8 @@ module VagrantPlugins
 
         def create_host_only_network(options)
           # Create the interface
-          interface = execute("hostonlyif", "create")
-          name = interface[/^Interface '(.+?)' was successfully created$/, 1]
+          execute("hostonlyif", "create") =~ /^Interface '(.+?)' was successfully created$/
+            name = $1.to_s
 
           # Configure it
           execute("hostonlyif", "ipconfig", name,
@@ -69,17 +69,15 @@ module VagrantPlugins
         def delete_unused_host_only_networks
           networks = []
           execute("list", "hostonlyifs").split("\n").each do |line|
-            if network = line[/^Name:\s+(.+?)$/, 1]
-              networks << network
-            end
+            networks << $1.to_s if line =~ /^Name:\s+(.+?)$/
           end
 
           execute("list", "vms").split("\n").each do |line|
-            if vm = line[/^".+?"\s+\{(.+?)\}$/, 1]
-              info = execute("showvminfo", vm, "--machinereadable", retryable: true)
-              info.split("\n").each do |line|
-                if adapter = line[/^hostonlyadapter\d+="(.+?)"$/, 1]
-                  networks.delete(adapter)
+            if line =~ /^".+?"\s+\{(.+?)\}$/
+              info = execute("showvminfo", $1.to_s, "--machinereadable", retryable: true)
+              info.split("\n").each do |inner_line|
+                if inner_line =~ /^hostonlyadapter\d+="(.+?)"$/
+                  networks.delete($1.to_s)
                 end
               end
             end
@@ -112,12 +110,12 @@ module VagrantPlugins
 
             if adapter[:hostonly]
               args.concat(["--hostonlyadapter#{adapter[:adapter]}",
-                          adapter[:hostonly]])
+                          adapter[:hostonly], "--cableconnected#{adapter[:adapter]}", "on"])
             end
 
             if adapter[:intnet]
               args.concat(["--intnet#{adapter[:adapter]}",
-                          adapter[:intnet]])
+                          adapter[:intnet], "--cableconnected#{adapter[:adapter]}", "on"])
             end
 
             if adapter[:mac_address]
@@ -168,7 +166,36 @@ module VagrantPlugins
           output = ""
           total = ""
           last  = 0
-          execute("import", ovf) do |type, data|
+
+          output = execute("import", "-n", ovf)
+          output =~ /Suggested VM name "(.+?)"/
+          suggested_name = $1.to_s
+          specified_name = "#{suggested_name}_#{(Time.now.to_f * 1000.0).to_i}_#{rand(100000)}" #Millisecond + Random
+
+          #Build the specified name param list
+          name_params = Array.new
+          name_params << "--vsys" << "0" << "--vmname" << specified_name
+
+          #Extract the disks list and build the disk target params
+          disk_params = Array.new
+          disks = output.scan(/(\d+): Hard disk image: source image=.+, target path=(.+),/)
+          disks.each do |unit_num, path|
+            disk_params << "--vsys"
+            disk_params << "0"  #Derive vsys num .. do we support OVF's with multiple machines?
+            disk_params << "--unit"
+            disk_params << unit_num
+            disk_params << "--disk"
+            if Vagrant::Util::Platform.windows?
+              # we use the block form of sub here to ensure that if the specified_name happens to end with a number (which is fairly likely) then
+              # we won't end up having the character sequence of a \ followed by a number be interpreted as a back reference.  For example, if
+              # specified_name were "abc123", then "\\abc123\\".reverse would be "\\321cba\\", and the \3 would be treated as a back reference by the sub
+              disk_params << path.reverse.sub("\\#{suggested_name}\\".reverse) { "\\#{specified_name}\\".reverse }.reverse # Replace only last occurrence
+            else
+              disk_params << path.reverse.sub("/#{suggested_name}/".reverse, "/#{specified_name}/".reverse).reverse # Replace only last occurrence
+            end
+          end
+
+          execute("import", ovf , *name_params, *disk_params) do |type, data|
             if type == :stdout
               # Keep track of the stdout so that we can get the VM name
               output << data
@@ -181,8 +208,8 @@ module VagrantPlugins
               if lines.include?("OK.")
                 # The progress of the import will be in the last line. Do a greedy
                 # regular expression to find what we're looking for.
-                if current = lines.last[/.+(\d{2})%/, 1]
-                  current = current.to_i
+                if lines.last =~ /.+(\d{2})%/
+                  current = $1.to_i
                   if current > last
                     last = current
                     yield current if block_given?
@@ -192,21 +219,19 @@ module VagrantPlugins
             end
           end
 
-          # Find the name of the VM name
-          name = output[/Suggested VM name "(.+?)"/, 1]
-          if !name
-            @logger.error("Couldn't find VM name in the output.")
-            return nil
-          end
-
           output = execute("list", "vms")
-          if existing_vm = output[/^"#{Regexp.escape(name)}" \{(.+?)\}$/, 1]
-            return existing_vm
+
+          if output =~ /^"#{Regexp.escape(specified_name)}" \{(.+?)\}$/
+            return $1.to_s
           end
 
           nil
         end
 
+        def max_network_adapters
+          36
+        end
+
         def read_forwarded_ports(uuid=nil, active_only=false)
           uuid ||= @uuid
 
@@ -218,19 +243,17 @@ module VagrantPlugins
           info.split("\n").each do |line|
             # This is how we find the nic that a FP is attached to,
             # since this comes first.
-            if nic = line[/^nic(\d+)=".+?"$/, 1]
-              current_nic = nic.to_i
-            end
+            current_nic = $1.to_i if line =~ /^nic(\d+)=".+?"$/
 
-            # If we care about active VMs only, then we check the state
-            # to verify the VM is running.
-            if active_only && (state = line[/^VMState="(.+?)"$/, 1] and state != "running")
-              return []
-            end
+              # If we care about active VMs only, then we check the state
+              # to verify the VM is running.
+              if active_only && line =~ /^VMState="(.+?)"$/ && $1.to_s != "running"
+                return []
+              end
 
             # Parse out the forwarded port information
-            if matcher = /^Forwarding.+?="(.+?),.+?,.*?,(.+?),.*?,(.+?)"$/.match(line)
-              result = [current_nic, matcher[1], matcher[2].to_i, matcher[3].to_i]
+            if line =~ /^Forwarding.+?="(.+?),.+?,.*?,(.+?),.*?,(.+?)"$/
+              result = [current_nic, $1.to_s, $2.to_i, $3.to_i]
               @logger.debug("  - #{result.inspect}")
               results << result
             end
@@ -244,14 +267,14 @@ module VagrantPlugins
             info = {}
 
             block.split("\n").each do |line|
-              if name = line[/^Name:\s+(.+?)$/, 1]
-                info[:name] = name
-              elsif ip = line[/^IPAddress:\s+(.+?)$/, 1]
-                info[:ip] = ip
-              elsif netmask = line[/^NetworkMask:\s+(.+?)$/, 1]
-                info[:netmask] = netmask
-              elsif status = line[/^Status:\s+(.+?)$/, 1]
-                info[:status] = status
+              if line =~ /^Name:\s+(.+?)$/
+                info[:name] = $1.to_s
+              elsif line =~ /^IPAddress:\s+(.+?)$/
+                info[:ip] = $1.to_s
+              elsif line =~ /^NetworkMask:\s+(.+?)$/
+                info[:netmask] = $1.to_s
+              elsif line =~ /^Status:\s+(.+?)$/
+                info[:status] = $1.to_s
               end
             end
 
@@ -260,21 +283,57 @@ module VagrantPlugins
           end
         end
 
+        def read_dhcp_servers
+          execute("list", "dhcpservers", retryable: true).split("\n\n").collect do |block|
+            info = {}
+
+            block.split("\n").each do |line|
+              if network = line[/^NetworkName:\s+HostInterfaceNetworking-(.+?)$/, 1]
+                info[:network]      = network
+                info[:network_name] = "HostInterfaceNetworking-#{network}"
+              elsif ip = line[/^IP:\s+(.+?)$/, 1]
+                info[:ip] = ip
+              elsif netmask = line[/^NetworkMask:\s+(.+?)$/, 1]
+                info[:netmask] = netmask
+              elsif lower = line[/^lowerIPAddress:\s+(.+?)$/, 1]
+                info[:lower] = lower
+              elsif upper = line[/^upperIPAddress:\s+(.+?)$/, 1]
+                info[:upper] = upper
+              end
+            end
+
+            info
+          end
+        end
+
         def read_guest_additions_version
           output = execute("guestproperty", "get", @uuid, "/VirtualBox/GuestAdd/Version",
                            retryable: true)
-          if value = output[/^Value: (.+?)$/, 1]
+          if output =~ /^Value: (.+?)$/
             # Split the version by _ since some distro versions modify it
             # to look like this: 4.1.2_ubuntu, and the distro part isn't
             # too important.
+            value = $1.to_s
             return value.split("_").first
           end
 
+          # If we can't get the guest additions version by guest property, try
+          # to get it from the VM info itself.
+          info = execute("showvminfo", @uuid, "--machinereadable", retryable: true)
+          info.split("\n").each do |line|
+            return $1.to_s if line =~ /^GuestAdditionsVersion="(.+?)"$/
+          end
+
           return nil
         end
 
         def read_guest_ip(adapter_number)
-          read_guest_property("/VirtualBox/GuestInfo/Net/#{adapter_number}/V4/IP")
+          ip = read_guest_property("/VirtualBox/GuestInfo/Net/#{adapter_number}/V4/IP")
+          if !valid_ip_address?(ip)
+            raise Vagrant::Errors::VirtualBoxGuestPropertyNotFound, guest_property: "/VirtualBox/GuestInfo/Net/#{adapter_number}/V4/IP"
+          end
+
+          return ip
         end
 
         def read_guest_property(property)
@@ -287,44 +346,21 @@ module VagrantPlugins
         end
 
         def read_host_only_interfaces
-          dhcp = {}
-          execute("list", "dhcpservers", retryable: true).split("\n\n").each do |block|
-            info = {}
-
-            block.split("\n").each do |line|
-              if network = line[/^NetworkName:\s+HostInterfaceNetworking-(.+?)$/, 1]
-                info[:network] = network
-              elsif ip = line[/^IP:\s+(.+?)$/, 1]
-                info[:ip] = ip
-              elsif lower = line[/^lowerIPAddress:\s+(.+?)$/, 1]
-                info[:lower] = lower
-              elsif upper = line[/^upperIPAddress:\s+(.+?)$/, 1]
-                info[:upper] = upper
-              end
-            end
-
-            # Set the DHCP info
-            dhcp[info[:network]] = info
-          end
-
           execute("list", "hostonlyifs", retryable: true).split("\n\n").collect do |block|
             info = {}
 
             block.split("\n").each do |line|
-              if name = line[/^Name:\s+(.+?)$/, 1]
-                info[:name] = name
-              elsif ip = line[/^IPAddress:\s+(.+?)$/, 1]
-                info[:ip] = ip
-              elsif netmask = line[/^NetworkMask:\s+(.+?)$/, 1]
-                info[:netmask] = netmask
-              elsif status = line[/^Status:\s+(.+?)$/, 1]
-                info[:status] = status
+              if line =~ /^Name:\s+(.+?)$/
+                info[:name] = $1.to_s
+              elsif line =~ /^IPAddress:\s+(.+?)$/
+                info[:ip] = $1.to_s
+              elsif line =~ /^NetworkMask:\s+(.+?)$/
+                info[:netmask] = $1.to_s
+              elsif line =~ /^Status:\s+(.+?)$/
+                info[:status] = $1.to_s
               end
             end
 
-            # Set the DHCP info if it exists
-            info[:dhcp] = dhcp[info[:name]] if dhcp[info[:name]]
-
             info
           end
         end
@@ -332,9 +368,7 @@ module VagrantPlugins
         def read_mac_address
           info = execute("showvminfo", @uuid, "--machinereadable", retryable: true)
           info.split("\n").each do |line|
-            if mac = line[/^macaddress1="(.+?)"$/, 1]
-              return mac
-            end
+            return $1.to_s if line =~ /^macaddress1="(.+?)"$/
           end
 
           nil
@@ -355,8 +389,8 @@ module VagrantPlugins
 
         def read_machine_folder
           execute("list", "systemproperties", retryable: true).split("\n").each do |line|
-            if folder = line[/^Default machine folder:\s+(.+?)$/i, 1]
-              return folder
+            if line =~ /^Default machine folder:\s+(.+?)$/i
+              return $1.to_s
             end
           end
 
@@ -367,21 +401,21 @@ module VagrantPlugins
           nics = {}
           info = execute("showvminfo", @uuid, "--machinereadable", retryable: true)
           info.split("\n").each do |line|
-            if matcher = /^nic(\d+)="(.+?)"$/.match(line)
-              adapter = matcher[1].to_i
-              type    = matcher[2].to_sym
+            if line =~ /^nic(\d+)="(.+?)"$/
+              adapter = $1.to_i
+              type    = $2.to_sym
 
               nics[adapter] ||= {}
               nics[adapter][:type] = type
-            elsif matcher = /^hostonlyadapter(\d+)="(.+?)"$/.match(line)
-              adapter = matcher[1].to_i
-              network = matcher[2].to_s
+            elsif line =~ /^hostonlyadapter(\d+)="(.+?)"$/
+              adapter = $1.to_i
+              network = $2.to_s
 
               nics[adapter] ||= {}
               nics[adapter][:hostonly] = network
-            elsif matcher = /^bridgeadapter(\d+)="(.+?)"$/.match(line)
-              adapter = matcher[1].to_i
-              network = matcher[2].to_s
+            elsif line =~ /^bridgeadapter(\d+)="(.+?)"$/
+              adapter = $1.to_i
+              network = $2.to_s
 
               nics[adapter] ||= {}
               nics[adapter][:bridge] = network
@@ -395,8 +429,8 @@ module VagrantPlugins
           output = execute("showvminfo", @uuid, "--machinereadable", retryable: true)
           if output =~ /^name="<inaccessible>"$/
             return :inaccessible
-          elsif state = output[/^VMState="(.+?)"$/, 1]
-            return state.to_sym
+          elsif output =~ /^VMState="(.+?)"$/
+            return $1.to_sym
           end
 
           nil
@@ -405,7 +439,9 @@ module VagrantPlugins
         def read_used_ports
           ports = []
           execute("list", "vms", retryable: true).split("\n").each do |line|
-            if uuid = line[/^".+?" \{(.+?)\}$/, 1]
+            if line =~ /^".+?" \{(.+?)\}$/
+              uuid = $1.to_s
+
               # Ignore our own used ports
               next if uuid == @uuid
 
@@ -429,12 +465,16 @@ module VagrantPlugins
           results
         end
 
+        def remove_dhcp_server(network_name)
+          execute("dhcpserver", "remove", "--netname", network_name)
+        end
+
         def set_mac_address(mac)
           execute("modifyvm", @uuid, "--macaddress1", mac)
         end
 
         def set_name(name)
-          execute("modifyvm", @uuid, "--name", name)
+          execute("modifyvm", @uuid, "--name", name, retryable: true)
         end
 
         def share_folders(folders)
@@ -443,13 +483,13 @@ module VagrantPlugins
               folder[:name],
               "--hostpath",
               folder[:hostpath]]
-            args << "--transient" if folder.has_key?(:transient) && folder[:transient]
-
-            # Add the shared folder
-            execute("sharedfolder", "add", @uuid, *args)
+            args << "--transient" if folder.key?(:transient) && folder[:transient]
 
             # Enable symlinks on the shared folder
             execute("setextradata", @uuid, "VBoxInternal2/SharedFoldersEnableSymlinksCreate/#{folder[:name]}", "1")
+
+            # Add the shared folder
+            execute("sharedfolder", "add", @uuid, *args)
           end
         end
 
@@ -510,6 +550,16 @@ module VagrantPlugins
           end
         end
 
+        def valid_ip_address?(ip)
+          # Filter out invalid IP addresses
+          # GH-4658 VirtualBox can report an IP address of 0.0.0.0 for FreeBSD guests.
+          if ip == "0.0.0.0"
+            return false
+          else
+            return true
+          end
+        end
+
         def verify!
           # This command sometimes fails if kernel drivers aren't properly loaded
           # so we just run the command and verify that it succeeded.
