@@ -73,6 +73,15 @@ module VagrantPlugins
       #
       # @return [Symbol]
       def state
+        # We have to check if the UID matches to avoid issues with
+        # VirtualBox.
+        uid = @machine.uid
+        if uid && uid.to_s != Process.uid.to_s
+          raise Vagrant::Errors::VirtualBoxUserMismatch,
+            original_uid: uid.to_s,
+            uid: Process.uid.to_s
+        end
+
         # Determine the ID of the state here.
         state_id = nil
         state_id = :not_created if !@driver.uuid
