@@ -24,13 +24,6 @@ module VagrantPlugins
           end
         end
 
-        if @machine.guest.capability?(:docker_configure_auto_start)
-          @machine.ui.detail(I18n.t("vagrant.docker_configure_autostart"))
-          @machine.guest.capability(:docker_configure_auto_start)
-        else
-          @machine.env.ui.warn I18n.t('vagrant.docker_auto_start_not_available')
-        end
-
         if @machine.guest.capability?(:docker_configure_vagrant_user)
           @machine.guest.capability(:docker_configure_vagrant_user)
         end
