@@ -11,7 +11,7 @@ describe Vagrant::Util::NetworkIP do
 
   describe "network address" do
     it "calculates it properly" do
-      klass.network_address("192.168.2.234", "255.255.255.0").should == "192.168.2.0"
+      expect(klass.network_address("192.168.2.234", "255.255.255.0")).to eq("192.168.2.0")
     end
   end
 end
