@@ -10,4 +10,10 @@ describe Vagrant::Util::Platform do
       expect(described_class.fs_real_path("c:/foo").to_s).to eql("C:/foo")
     end
   end
+
+  describe "#windows_unc_path" do
+    it "correctly converts a path" do
+      expect(described_class.windows_unc_path("c:/foo").to_s).to eql("\\\\?\\c:\\foo")
+    end
+  end
 end
